// src/node/app/http/errors.ts
function httpError(message, init, data) {
  throw new HTTPError(message, init, data);
}
var HTTPError = class extends Error {
  constructor(message, init, data) {
    super(message);
    this.init = init;
    this.data = data;
  }
};
function isHTTPError(error) {
  return error instanceof HTTPError;
}

// src/node/app/http/request.ts
async function handleRequest(request, urlPattern, getClientAddress, loader, methods) {
  var _a;
  const methodOverride = request.method === "POST" ? (await request.formData()).get("_method") : null;
  const method = typeof methodOverride === "string" ? methodOverride : request.method;
  if (methods && !methods.includes(method))
    throw httpError("not found", 404);
  const url = new URL(request.url);
  if (!urlPattern.test(url))
    throw httpError("not found", 404);
  const mod = await loader();
  let handler = mod[method];
  if (!handler && method === "HEAD")
    handler = mod.GET;
  if (!handler)
    handler = mod.ANY;
  if (!handler) {
    throw httpError(`${method} method not allowed`, {
      status: 405,
      headers: {
        allow: getAllowedMethods(mod).join(", ")
      }
    });
  }
  const headers = {};
  const cookies = [];
  const params = ((_a = urlPattern.exec(url)) == null ? void 0 : _a.pathname.groups) ?? {};
  const response = await handler(
    createRequestEvent(
      request,
      url,
      params,
      getClientAddress,
      headers,
      cookies
    )
  );
  if (!(response instanceof Response)) {
    throw new Error(
      `Invalid return value from route handler at ${url.pathname}. Should return a \`Response\`.`
    );
  }
  for (const name in headers) {
    const value = headers[name];
    if (value !== null)
      response.headers.set(name, value + "");
  }
  for (const cookie of cookies) {
    response.headers.append("set-cookie", cookie);
  }
  return response;
}
function createRequestEvent(request, url, params, getClientAddress, headers = {}, cookies = []) {
  return {
    get url() {
      return url;
    },
    get clientAddress() {
      const address = getClientAddress();
      if (typeof address !== "string") {
        throw new Error("Could not determine `clientAddress`");
      }
      return address;
    },
    get params() {
      return params;
    },
    get request() {
      return request;
    },
    setHeaders(newHeaders) {
      for (const key in newHeaders) {
        const name = key.toLowerCase();
        const value = newHeaders[key];
        if (name === "set-cookie") {
          const newCookies = Array.isArray(value) ? value : value ? [value + ""] : [];
          for (const cookie of newCookies) {
            if (cookies.includes(cookie)) {
              throw new Error(
                `"${key}" header already has cookie with same value`
              );
            }
            cookies.push(cookie);
          }
        } else if (name in headers) {
          throw new Error(`"${key}" header is already set`);
        } else if (typeof value !== "string") {
          throw new Error(
            `"${key}" value must be a string (found ${typeof value}).`
          );
        } else {
          headers[name] = value;
        }
      }
    }
  };
}
function getAllowedMethods(mod) {
  const allowed = [];
  for (const method of HTTP_METHODS) {
    if (method in mod)
      allowed.push(method);
  }
  if (mod.GET || mod.HEAD)
    allowed.push("HEAD");
  return allowed;
}

// src/node/app/http/response.ts
var json = (data, init = {}) => {
  const responseInit = typeof init === "number" ? { status: init } : init;
  const headers = new Headers(responseInit.headers);
  if (!headers.has("Content-Type")) {
    headers.set("Content-Type", "application/json; charset=utf-8");
  }
  return new Response(JSON.stringify(data), {
    ...responseInit,
    headers
  });
};

// src/node/app/http/http.ts
function handleHTTPError(error) {
  if (isHTTPError(error)) {
    return json(
      { error: { message: error.message, data: error.data } },
      error.init
    );
  } else {
    return json({ error: { message: "internal server error" } }, 500);
  }
}
var HTTP_METHODS = /* @__PURE__ */ new Set([
  "GET",
  "POST",
  "PUT",
  "PATCH",
  "DELETE"
]);

// src/node/app/http/vercel.ts
function createEdgeRequestHandler(urlPattern, loader, options) {
  return async (request) => {
    try {
      return await handleRequest(
        request,
        urlPattern,
        () => request.headers.get("x-forwarded-for"),
        loader,
        options == null ? void 0 : options.methods
      );
    } catch (error) {
      return handleHTTPError(
        isHTTPError(error) ? error : httpError("internal server error", 500)
      );
    }
  };
}
export {
  createEdgeRequestHandler
};
